"""
******************************************************************************
This file is part of the lemmagen library. It gives support for lemmatization.
Copyright (C) 2011 Jernej Virag <jernej@virag.si>

The lemmagen library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************
"""

import os
import sys
import sysconfig

import lemmagen
from lemmagen import libLemmagen

IS_PYTHON3 = sys.version_info > (3, )

class Lemmatizer(object):

    def __init__(self, dictionary=lemmagen.DICTIONARY_SLOVENE, library_path=None):
        """
        @param dictionary: Path to lemmatizer dictionary file
        """

        this_dir = os.path.dirname(os.path.realpath(__file__))
        if not dictionary.startswith("/"):
            dictionary = os.path.join(this_dir, dictionary)

        self.lemmagen = libLemmagen.Lemmagen(bytes(dictionary, "UTF-8") if IS_PYTHON3 else dictionary)

    def lemmatize(self, word):
        if word is None:
            return None

        if IS_PYTHON3:
            return self._lemmatize_py3(word)
        else:
            return self._lemmatize_py2(word)

    def _lemmatize_py3(self, word):
        word_b = bytes(word, 'UTF-8')
        return str(self.lemmagen.lemmatize(word_b), 'UTF-8')

    def _lemmatize_py2(self, word):
        is_unicode = isinstance(word, unicode)
        if is_unicode:
            word = word.encode('utf-8')
        return self.lemmagen.lemmatize(word).decode('utf-8') if is_unicode else self.lemmagen.lemmatize(word)



# cython: language_level=3

"""
******************************************************************************
This file is part of the lemmagen library. It gives support for lemmatization.
Copyright (C) 2011 Jernej Virag <jernej@virag.si>

The lemmagen library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************
"""

from libc.string cimport strcpy, strlen
from libc.stdlib cimport malloc

cdef extern from "lemmagen.h":
    int lem_load_language_library(const char *file_name)
    void lem_lemmatize_word(const char *input_word, char *output_word)

cdef class Lemmagen:
    cdef char* output_buffer;

    STATUS_OK = 0
    STATUS_FILE_NOT_FOUND = -1

    def __init__(self, language_library_path):
        self.output_buffer = <char*> malloc(256 * sizeof(char))
        cdef int result = lem_load_language_library(language_library_path)
        if result == self.STATUS_FILE_NOT_FOUND:
            raise IOError("Lemmatizer dictionary file not found!")
        if not result == self.STATUS_OK:
            raise IOError("Failed to initialize lemmatizer!")

    def lemmatize(self, input_buffer: bytes) -> bytes:
        lem_lemmatize_word(input_buffer, self.output_buffer)
        return self.output_buffer


from setuptools import setup, Extension
import os
import platform

root_dir = os.path.dirname(os.path.realpath(__file__))
src_dir = os.path.join(root_dir, "lemmagen/lib/src")
include_dir = os.path.join(root_dir, "lemmagen/lib/include")

# Setup compilation arguments for native code
compile_args = []
link_args = []
if os.name == 'posix':
  compile_args = ["-O3", "-ffunction-sections", "-fdata-sections", "-fvisibility-inlines-hidden"]
  if platform.system() == 'Darwin':
    link_args = ["-Wl,-dead_strip"]
  else:
    compile_args.append("-std=c++11")
    link_args = ["-Wl,-z,noexecstack", "-Wl,-z,now", "-Wl,-z,relro", "-Wl,--gc-sections"]
elif os.name == 'nt':
  compile_args = ["/Ox", "/EHsc"]
  link_args = []

lib = Extension('lemmagen.libLemmagen',
                ["lemmagen/libLemmagen.pyx", os.path.join(src_dir, "lemmagen.cpp"), os.path.join(src_dir, "RdrLemmatizer.cpp")],
                extra_compile_args=compile_args,
                extra_link_args=link_args,
                include_dirs=[include_dir])

setup(name="Lemmagen",
      version="1.3.2",
      description="LemmaGen lemmatizer for Python supporing Slovene, Serbian, Romanian, Estonian, Bulgarian and other languages",
      package_data={'lemmagen':["lib/data/*"]},
      license="GPLv2+",
      author="Jernej Virag",
      author_email="jernej@virag.si",
      setup_requires = ["setuptools>=18.0","cython>=0.28.4"],
      classifiers=[
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Development Status :: 5 - Production/Stable",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Topic :: Text Processing :: Linguistic",
      ],
      ext_modules=[lib],
      test_suite="tests",
      packages=["lemmagen"])



"""
******************************************************************************
This file is part of the lemmagen library. It gives support for lemmatization.
Copyright (C) 2011 Jernej Virag <jernej@virag.si>

The lemmagen library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************
"""

DICTIONARY_SLOVENE="lib/data/lem-me-sl.bin"
DICTIONARY_ENGLISH="lib/data/lem-m-en.bin"
DICTIONARY_SERBIAN="lib/data/lem-me-sr.bin"
DICTIONARY_ITALIAN="lib/data/lem-m-it.bin"
DICTIONARY_ROMANIAN="lib/data/lem-me-ro.bin"
DICTIONARY_HUNGARIAN="lib/data/lem-me-hu.bin"
DICTIONARY_FRENCH="lib/data/lem-me-fr.bin"
DICTIONARY_GERMAN="lib/data/lem-m-ge.bin"
DICTIONARY_SPANISH="lib/data/lem-m-sp.bin"
DICTIONARY_CZECH="lib/data/lem-me-cs.bin"
DICTIONARY_BULGARIAN="lib/data/lem-me-bg.bin"
DICTIONARY_ESTONIAN="lib/data/lem-me-et.bin"